__author__ = 'sscoble'

import inspect
from di import ServiceStore

class Servicer(object):
    def __init__(self, id, store=ServiceStore):
        self.id = id
        self.store = store

    def __call__(self, f):
        if inspect.isclass(f):
            self.store.add_service(self.id, f())
        else:
            self.store.add_service(self.id, f)

        return f

class Injector(object):
    def __init__(self, list_of_service_ids, store=ServiceStore):
        self.ids = list_of_service_ids
        self.store = store

    def __call__(self, f):
        if 'self' in inspect.getargspec(f).args:
            def new_f(s):
                return f(s, *self.get_services())
            return new_f
        else:
            def new_f():
                return f(*self.get_services())
            return new_f

    def get_services(self):
        for id in self.ids:
            yield self.store.get_service(id)

service = Servicer
inject = Injector


from unittest import TestCase
from di.decorator import service, inject
from di import ServiceStore

class TestDecorators(TestCase):

    def test_function_service_added(self):
        actual = ServiceStore.get_service('sf').__name__
        self.assertEqual('func', actual, "Service not loaded properly.  Expecting: 'func', got: " + actual)

    def test_class_service_added(self):
        actual = ServiceStore.get_service('sc').__class__
        self.assertEqual(klass, actual, "Service not loaded properly.  Expecting: '" + repr(klass) + "', got: '" + repr(actual) + "'")

    def test_inject_works_on_function(self):
        self.assertEqual('me', clone(), "Injection failed.")

    def test_inject_works_on_init(self):
        i = injected()
        actual_class = i.get_k().__class__
        self.assertEqual(klass, actual_class, "Injection failed for class service.")

        actual_func = i.get_f().__name__
        self.assertEqual('func', actual_func, "Injection failed for function service.")

    def test_injected_service_works(self):
        i = ServiceStore.get_service('sn')
        actual_class = i.get_k().__class__
        self.assertEqual(klass, actual_class, "Injection failed for class service.")

        actual_func = i.get_f().__name__
        self.assertEqual('func', actual_func, "Injection failed for function service.")


@service('sf')
def func(arg):
    return arg

@service('sc')
class klass(object): pass

@inject(['sf'])
def clone(f):
    return f('me')

class injected(object):
    @inject(['sc','sf'])
    def __init__(self, k, f):
        self.k = k
        self.f = f

    def get_k(self):
        return self.k

    def get_f(self):
        return self.f

@service('sn')
class injected_service(object):
    @inject(['sc','sf'])
    def __init__(self, k, f):
        self.k = k
        self.f = f

    def get_k(self):
        return self.k

    def get_f(self):
        return self.f



__author__ = 'sscoble'

from exceptions import ServiceNotFound

class ServiceStore(object):
    services = {}

    @classmethod
    def add_service(cls, id, service):
        cls.services[id] = service

    @classmethod
    def get_service(cls, id):
        if id in cls.services.keys():
            return cls.services[id]
        else:
            raise ServiceNotFound("Service " + id + " not found.")



__author__ = 'sscoble'

class BaseDIException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class ServiceNotFound(BaseDIException): pass


from distutils.core import setup

setup(
    name='CNSdi',
    version='0.1.0',
    author='Scott Scoble',
    author_email='sscoble@codesmith.ws',
    packages=['di','di.test'],
    license='LICENSE.txt',
    description='Simple Dependency Injection for python',
    long_description=open('README.txt').read(),
    install_requires=[]
)